Index: fluid/Fl_Type.h
===================================================================
--- fluid/Fl_Type.h	(revision 11804)
+++ fluid/Fl_Type.h	(working copy)
@@ -31,6 +31,7 @@
 #include "Fluid_Image.h"
 #include <FL/fl_draw.H>
 #include <stdarg.h>
+#include "ExternalCodeEditor.h"
 
 void set_modflag(int mf);
 
@@ -109,7 +110,7 @@
   virtual void open();	// what happens when you double-click
 
   // read and write data to a saved file:
-  void write();
+  virtual void write();
   virtual void write_properties();
   virtual void read_property(const char *);
   virtual int read_fdesign(const char*, const char*);
@@ -143,6 +144,7 @@
   virtual int is_menu_button() const;
   virtual int is_group() const;
   virtual int is_window() const;
+  virtual int is_code() const;
   virtual int is_code_block() const;
   virtual int is_decl_block() const;
   virtual int is_comment() const;
@@ -185,15 +187,66 @@
 };
 
 class Fl_Code_Type : public Fl_Type {
+  ExternalCodeEditor editor_;
 public:
   Fl_Type *make();
+  void write();
   void write_code1();
   void write_code2();
   void open();
   virtual const char *type_name() {return "code";}
   int is_code_block() const {return 0;}
+  int is_code() const {return 1;}
   int pixmapID() { return 8; }
   virtual int is_public() const;
+  // See if external editor is open
+  int is_editing() {
+    return editor_.is_editing();
+  }
+  // Reap the editor's pid
+  // Returns:
+  //   -2 -- editor not open
+  //   -1 -- waitpid() failed (errno has reason)
+  //    0 -- process still running
+  //   >0 -- process finished + reaped (value is pid)
+  int reap_editor() {
+    return editor_.reap_editor();
+  }
+  // Handle external editor file modifications
+  //    Returns 1 if so, 0 if not, -1 on error.
+  //    If changed, record keeping is updated and file's contents is loaded into ram
+  //
+  int handle_changes() {
+    int changed = 0;
+    if ( editor_.is_editing() ) {
+      switch ( editor_.handle_changes() ) {
+        case 0: { // unchanged
+          break;
+        }
+        case 1: { // changed
+          ++changed;
+          // TODO: Figure out how saving a fluid file
+          //       can be intercepted to grab current contents
+          //       of editor file..
+          //
+          const char *s = 0;
+          switch ( editor_.get_code(&s) ) {
+            case 0:             // Success
+              name(s);          // update value in ram
+              free((void*)s);   // free string get_code() returns
+              break;
+            case -1:            // Failed: couldn't read file (dialog showed reason)
+              return(-1);       // return, something is very wrong
+          }
+	  break;
+        }
+        case -1: {    // error
+          break;      // shrug - probably dont want to post a dialog
+        }
+      }
+    }
+    return changed;
+  }
 };
 
 class Fl_CodeBlock_Type : public Fl_Type {
Index: fluid/ExternalCodeEditor_WIN32.h
===================================================================
--- fluid/ExternalCodeEditor_WIN32.h	(revision 0)
+++ fluid/ExternalCodeEditor_WIN32.h	(revision 0)
@@ -0,0 +1,38 @@
+#include <windows.h>    // CreateFile()..
+#include <string.h>     // sprintf()..
+
+//
+// External code editor class for Windows
+//
+// TODO: For now, a completely separate implimentation from unix.
+//       Once the code is stable, I'll merge any common code.
+//
+
+class ExternalCodeEditor {
+  PROCESS_INFORMATION pinfo_; // CreateProcess() handle to running process 
+  const char *filename_;
+  FILETIME      file_mtime_;  // last modify time of the file (used to determine if file changed)
+  LARGE_INTEGER file_size_;   // last file size (used to determine if changed)
+protected:
+  void kill_editor();
+  const char *create_tmpdir();
+  const char *tmp_filename();
+  int make_args(char *s, int *aargc, char ***argv);
+  int start_editor(const char *cmd, const char *filename);
+  void set_filename(const char *val);
+public:
+  ExternalCodeEditor();
+  ~ExternalCodeEditor();
+  int is_editing();
+  DWORD reap_editor();
+  const char *filename() { return filename_; }
+  int open_editor(const char *editor_cmd, const char *code);
+  int get_code(const char **code);
+  int handle_changes();
+  int remove_tmpfile();
+  // Public static methods
+  static const char* tmpdir_name();
+  static void tmpdir_clear();
+  static int editors_open();
+  static void set_update_timer_callback(Fl_Timeout_Handler);
+};
Index: fluid/alignment_panel.fl
===================================================================
--- fluid/alignment_panel.fl	(revision 11804)
+++ fluid/alignment_panel.fl	(working copy)
@@ -46,6 +46,12 @@
 decl {extern int show_comments;} {public local
 }
 
+decl {extern int G_use_external_editor;} {public local
+}
+
+decl {extern char G_external_editor_command[512];} {public local
+}
+
 decl {extern int show_coredevmenus;} {public local
 }
 
@@ -58,8 +64,7 @@
 decl {Fl_Text_Buffer *shell_run_buffer;} {public local
 }
 
-Function {make_project_window()} {open
-} {
+Function {make_project_window()} {} {
   Fl_Window project_window {
     label {Project Settings} open
     xywh {396 475 399 252} type Double hide
@@ -129,7 +134,7 @@
         }
         Fl_Input i18n_include_input {
           label {\#include:}
-          callback i18n_text_cb selected
+          callback i18n_text_cb
           tooltip {The include file for internationalization.} xywh {100 78 272 20} box THIN_DOWN_BOX labelfont 1 textfont 4
         }
         Fl_Input i18n_file_input {
@@ -158,10 +163,11 @@
 decl {void scheme_cb(Fl_Choice *, void *);} {public local
 }
 
-Function {make_settings_window()} {} {
+Function {make_settings_window()} {open
+} {
   Fl_Window settings_window {
     label {GUI Settings} open
-    xywh {393 191 349 241} type Double hide non_modal
+    xywh {355 85 360 355} type Double non_modal visible
   } {
     Fl_Choice scheme_choice {
       label {Scheme: }
@@ -194,17 +200,17 @@
       }
     }
     Fl_Group {} {open
-      xywh {116 43 220 126} labelfont 1 align 0
+      xywh {20 43 330 161} labelfont 1 align 0
     } {
       Fl_Box {} {
         label {Options: }
-        xywh {140 43 1 25} labelfont 1 align 4
+        xywh {139 43 1 25} labelfont 1 align 4
       }
       Fl_Check_Button tooltips_button {
         label {Show Tooltips}
         callback {Fl_Tooltip::enable(tooltips_button->value());
 fluid_prefs.set("show_tooltips", tooltips_button->value());}
-        xywh {138 43 113 25} down_box DOWN_BOX labelsize 12
+        xywh {137 43 113 25} down_box DOWN_BOX labelsize 12
         code0 {int b;}
         code1 {fluid_prefs.get("show_tooltips", b, 1);}
         code2 {tooltips_button->value(b);}
@@ -213,7 +219,7 @@
       Fl_Check_Button completion_button {
         label {Show Completion Dialogs}
         callback {fluid_prefs.set("show_completion_dialogs", completion_button->value());}
-        xywh {138 68 186 25} down_box DOWN_BOX labelsize 12
+        xywh {137 68 186 25} down_box DOWN_BOX labelsize 12
         code0 {int b;}
         code1 {fluid_prefs.get("show_completion_dialogs", b, 1);}
         code2 {completion_button->value(b);}
@@ -221,7 +227,7 @@
       Fl_Check_Button openlast_button {
         label {Open Previous File on Startup}
         callback {fluid_prefs.set("open_previous_file", openlast_button->value());}
-        xywh {138 93 214 25} down_box DOWN_BOX labelsize 12
+        xywh {137 93 213 25} down_box DOWN_BOX labelsize 12
         code0 {int b;}
         code1 {fluid_prefs.get("open_previous_file", b, 0);}
         code2 {openlast_button->value(b);}
@@ -229,7 +235,7 @@
       Fl_Check_Button prevpos_button {
         label {Remember Window Positions}
         callback {fluid_prefs.set("prev_window_pos", prevpos_button->value());}
-        xywh {138 118 209 25} down_box DOWN_BOX labelsize 12
+        xywh {137 117 209 25} down_box DOWN_BOX labelsize 12
         code0 {int b;}
         code1 {fluid_prefs.get("prev_window_pos", b, 1);}
         code2 {prevpos_button->value(b);}
@@ -239,7 +245,7 @@
         callback {show_comments = show_comments_button->value();
 fluid_prefs.set("show_comments", show_comments);
 redraw_browser();}
-        xywh {138 143 209 25} down_box DOWN_BOX labelsize 12
+        xywh {137 141 209 25} down_box DOWN_BOX labelsize 12
         code1 {fluid_prefs.get("show_comments", show_comments, 1);}
         code2 {show_comments_button->value(show_comments);}
       }
@@ -248,16 +254,44 @@
       label {\# Recent Files: }
       callback {fluid_prefs.set("recent_files", recent_spinner->value());
 load_history();}
-      xywh {140 173 40 25} labelfont 1 when 1
+      xywh {138 170 40 25} labelfont 1 when 1
       code0 {int c;}
       code1 {fluid_prefs.get("recent_files", c, 5);}
       code2 {recent_spinner->maximum(10);}
       code3 {recent_spinner->value(c);}
     }
+    Fl_Group {} {open
+      xywh {10 210 337 95} box THIN_UP_BOX color 47
+    } {
+      Fl_Check_Button use_external_editor_button {
+        label {Use external editor?}
+        callback {G_use_external_editor = use_external_editor_button->value();
+fluid_prefs.set("use_external_editor", G_use_external_editor);
+redraw_browser();}
+        xywh {25 218 209 22} down_box DOWN_BOX labelsize 12
+        code1 {fluid_prefs.get("use_external_editor", G_use_external_editor, 0);}
+        code2 {use_external_editor_button->value(G_use_external_editor);}
+      }
+      Fl_Input editor_command_input {
+        label {Editor Command}
+        callback {strncpy(G_external_editor_command, editor_command_input->value(), sizeof(G_external_editor_command)-1);
+G_external_editor_command[sizeof(G_external_editor_command)-1] = 0;
+fluid_prefs.set("external_editor_command", G_external_editor_command);
+redraw_browser();}
+        tooltip {The editor command to open your external text editor.
+Include any necessary flags to ensure your editor does not background itself.
+Examples:
+    gvim -f
+    gedit
+ emacs} xywh {25 264 305 21} labelsize 10 align 5 when 1 textsize 10
+        code1 {fluid_prefs.get("external_editor_command", G_external_editor_command, "", sizeof(G_external_editor_command)-1);}
+        code2 {editor_command_input->value(G_external_editor_command);}
+      }
+    }
     Fl_Button {} {
       label Close
-      callback {settings_window->hide();}
-      tooltip {Close this dialog.} xywh {276 205 64 25}
+      callback {settings_window->hide();} selected
+      tooltip {Close this dialog.} xywh {285 320 64 25}
     }
   }
 }
Index: fluid/README-ExternalCodeEditor.txt
===================================================================
--- fluid/README-ExternalCodeEditor.txt	(revision 0)
+++ fluid/README-ExternalCodeEditor.txt	(revision 0)
@@ -0,0 +1,232 @@
+
+    When user double-clicks code, the external editor is opened (if not already).
+    On open, we first see if an editor is already open that was recently closed.
+    If recently closed, we reap it first.
+    If not closed, post a dialog warning an editor is already open.
+    Once we're sure no editor is already open, we save the Code contents
+    in ram to a temp file, update our record of the file's mtime and file size,
+    then start the editor process. And if there's no "update" timer is running,
+    we start one.
+
+    When an editor is closed, the "update" timer should reap it; this timer
+    runs every 2 seconds, checking if editors are open, and if so, if any files
+    have changed. And if changed, we load the file into ram. And if the editor
+    exited, we reap its process ID, and remove the temp file.
+
+    The closing editor doesn't have to be reaped /immediately/,
+    since any kind of save or quit can check for the temp file
+    and if it exists, check if the pid exited, and if so, reap it.
+
+    So when e.g. a Save() is done, after grabbing temp file into ram,
+    if pid reaps (using waitpid()), remove the temp file.
+
+    TEMP FILE
+	Create a temp dir for this instance of fluid (e.g. /tmp/.fluid-<PID>)
+	and save all related files there.
+
+	Files could have the name of the app, and the instance number
+	of the ExternalCodeEditor, e.g. /tmp/.fluid-<PID>/foo-123
+	Don't use the name of the function, as the user might change that,
+	and we don't want to have re-open the editor just because the temp file name changed.
+
+    FLUID OPENS .fl FILE
+	Close all existing open editors (kill them) if any
+	Create editor instances, but don't open the editors 
+	(even if saved open).
+
+    FLUID SAVES OR INVOKES "WRITE CODE"
+	The code instances are recursively called to generate code.
+	Each one will open the file being edited and load it into ram
+	before generating code.
+
+    FLUID USED TO OPEN AN EDITOR
+	Make sure editor doesn't have an unreaped temp file before
+	opening a new editor. If it does, load it FIRST, then invoke
+	the 'open editor' code, which should:
+
+	   a) Use a unique name to write a new temp file
+	   b) Open editor in background to edit the file
+	   c) Keep the pid / process handle so it can be reaped
+
+    SOMEONE CLOSES AN EDITOR
+
+	The app's timer periodically checks for reaped editors,
+	as well as any changed files (which are loaded and the "modified"
+	state of the editor is changed).
+
+    SOMEONE CLOSES FLUID
+
+	When fluid is closed, the update timer should be disabled.
+	
+	The currently loaded tree needs to be destroyed so that all the
+	ExternalCodeEditor destructors are called, which kills the open
+	editors and clears the temp files.
+
+	Then fluid should rmdir(2) the /tmp/.fluid-<PID> temp directory,
+	which should now be empty of all files.
+
+
+--------------
+-------------- 
+--------------
+
+IDEAS / WIP:
+
+    TIMEOUT POLL FOR MODIFIED FILES
+    -------------------------------
+    Have an Fl::add_timeout() that regularly checks the datestamps on
+    all external editor files, and if changed since the last check,
+    change fluid's modflag, so that it knows a "File -> Save" is needed.
+
+	XXX We probably *dont* need to load the file into ram, as any operation
+	XXX that needs that info will be calling write_code1(), which loads this
+	XXX content automatically.
+	 ^
+	/|\
+	 |
+	Actually, we DO have to load the file into ram if we want the tree
+	to update with the code changes (tree shows first line of code, so
+	if that changes, tree won't show it unless loaded)
+
+    Sucking in a half-written file during a *build* is not fatal,
+    but doing that during a Ctrl-S (to save the .fl file) would
+    certainly be bad, as the corrupt code would then be committed.
+
+    To avoid this, we should probably see if the mtime on the file
+    is less than 2 seconds, and if so, sleep until the mtime is at
+    least 2 seconds into the past.
+
+	XXX Another thing would be to include a "//FLUID-EOF:<hex-hash>" marker
+	XXX at the end of the file. We can look for this EOF mark when loading
+	XXX the file, and if not found, or found in the middle, we know something
+	XXX is wrong, and we should reload.
+	XXX 
+	XXX We should only expect the line to be at the exact end of the file,
+	XXX and no where else. We could retry at least 3 times (1 sec delay)
+	XXX until we succeed. If all three fail, throw an alert dialog.
+
+    SEE RFC POST ON FLTK.COREDEV on 06/29/16
+
+    ***
+
+    When an external editor saves a file, need to notice timestamp||filesize
+    change, and atomically:
+
+        o record the new timestamp/filesize
+
+	o Load the changes from the file into ram
+
+	o If filesize changes during load (read() != filesize stat),
+	  then redo record of filesize/timestamp, and reload the file
+	  until these three values agree with each other. (Editor
+	  may have been in the middle of saving while fluid updates)
+
+	o Make sure the 'modified' state of the .fl file is changed
+
+	o Make sure the 'auto refresh' for "Show Source Code" is triggered
+    
+    I guess this whole idea of an external editor has the possibility
+    of introducing partially written code into the IDE if the IDE happens
+    to load the file while the editor is still writing it out.
+
+    Not sure how to prevent this, unless the editor has a feature that
+    does what e.g. rdist/rsync does, namely write to a temp file, then
+    rename the completely written file in place of the editted file
+    to make the save operation atomic.
+
+External Code Editor Operation
+------------------------------
+When user double-clicks 'code', it first saves a temp file version
+of the code in memory, and forks an external editor to run in the background.
+The PID of that process is managed. When the process completes, it's assumed
+the editor has been closed. (This is why it's important the editor not
+background /itself/; we background it and keep an eye on the PID)
+
+The editor can remain open for the entire duration of the fluid edit,
+or can be closed and reopened again. While the file is open, hot edits to
+the file affect write code / build operations in fluid. Fluid regularly
+polls the file's modify timestamp and file size to determine if there's
+been a change, so that fluid can know when the state of the program is
+'modified' (e.g. 'File -> Save' is not grayed out, and the "Source Viewer"
+refresh knows to load new info saved from the editor)
+
+When an editor is closed, the process is reaped by fluid's main thread,
+sucking the temp file's contents into ram, and removes the temp file.
+
+When fluid exits, it kills any editors that are still open, and recursively
+removes the .fluid-<PID> temp directory created for it.
+
+If you try to open an editor for code that already has an editor open,
+an fl_alert() tells you the edit is open and indicates the filename
+(so you can locate it amongst other editor windows by the window title)
+
+The closing editor doesn't have to be reaped /immediately/,
+since any kind of save or quit can check for the temp file
+and if it exists, load it and check if the pid is still active,
+and if not, the process can be reaped and the temp file removed.
+
+THE TEMP FILE
+-------------
+    When you open the code editor with the external editor option enabled,
+    the current code is saved to a unique temp file, and the editor is
+    opened to edit that file.
+
+    First, a temp dir for the current fluid session is created (if it
+    doesn't already exist), e.g. /tmp/.fluid-<PID>/
+    and then any temp files are saved in that dir.
+    In this way multiple code entries can be open at a time with
+    live editors.
+
+    Files must have unique enough names to prevent overlap.
+    e.g. /tmp/.fluid-<PID>/1234abcd
+
+    It's tempting to use the name of the function in the filename,
+    but this gets complicated to manage, e.g. if the editor is open
+    and the user changes the name of the function in the fluid tree.
+    (The editor would have to be closed and the temp file renamed)
+    Better to use unique names that are not dependent on the function name..?
+
+    Either that, or changing the function name should first warn the user
+    needs to close and reopen the editor to see the name change.
+
+WHEN FLUID OPENS AN .fl FILE
+----------------------------
+    If a file is currently being edited, and fluid is told to
+    open another file, the destructors should handle closing all
+    open editors (kill them).
+
+    When a .fl file is loaded, no editor windows will be opened
+    automatically, even if the .fl file was saved while editors
+    were open.
+
+WHEN SOMEONE CHOOSES "File -> Save" or "File -> Write Code"
+-----------------------------------------------------------
+    The code instances are recursively called to generate code.
+    Each one will open the file being edited and load it into ram
+    before generating code.
+
+FLUID OPENS AN EXTERNAL EDITOR
+------------------------------
+   a) Write the code to a new temp file
+   b) Open the editor as a background process
+   c) Keep the pid / process handle so it can be reaped
+   d) Start a thread (if none already) to watch for reaped processes,
+      and search for the reaped pid in all the ExternalCodeEditor
+      instances until it's found, then call that instance's reap(pid)
+
+   Due to (d), we should probably have the ctor/dtor automatically
+   manage a list of pointers to ExternalCodeEditor instances 
+   so that fluid doesn't have to walk the entire hierarchy
+   looking for them, and can call one static ExternalCodeEditor
+   function to handle reaping a process.
+
+SOMEONE CLOSES AN EDITOR
+------------------------
+   fluid's main thread should handle checking for reaped processes
+   from time to time, and if one is found:
+   
+   	a) Load the temp file's current state into ram
+	b) Send the exit code / pid to the ExternalCodeEditor instance..
+	c) ..which should remove the temp file
+	d) ..reset the pid -> -1, and free/null out the temp filename
+
Index: fluid/alignment_panel.cxx
===================================================================
--- fluid/alignment_panel.cxx	(revision 11804)
+++ fluid/alignment_panel.cxx	(working copy)
@@ -197,12 +197,29 @@
 load_history();
 }
 
+Fl_Check_Button *use_external_editor_button=(Fl_Check_Button *)0;
+
+static void cb_use_external_editor_button(Fl_Check_Button*, void*) {
+  G_use_external_editor = use_external_editor_button->value();
+fluid_prefs.set("use_external_editor", G_use_external_editor);
+redraw_browser();
+}
+
+Fl_Input *editor_command_input=(Fl_Input *)0;
+
+static void cb_editor_command_input(Fl_Input*, void*) {
+  strncpy(G_external_editor_command, editor_command_input->value(), sizeof(G_external_editor_command)-1);
+G_external_editor_command[sizeof(G_external_editor_command)-1] = 0;
+fluid_prefs.set("external_editor_command", G_external_editor_command);
+redraw_browser();
+}
+
 static void cb_Close1(Fl_Button*, void*) {
   settings_window->hide();
 }
 
 Fl_Double_Window* make_settings_window() {
-  { settings_window = new Fl_Double_Window(349, 241, "GUI Settings");
+  { settings_window = new Fl_Double_Window(360, 355, "GUI Settings");
     { scheme_choice = new Fl_Choice(140, 10, 115, 25, "Scheme: ");
       scheme_choice->down_box(FL_BORDER_BOX);
       scheme_choice->labelfont(1);
@@ -213,14 +230,14 @@
       scheme_choice->value(s);
       scheme_cb(0, 0);
     } // Fl_Choice* scheme_choice
-    { Fl_Group* o = new Fl_Group(116, 43, 220, 126);
+    { Fl_Group* o = new Fl_Group(20, 43, 330, 161);
       o->labelfont(1);
       o->align(Fl_Align(FL_ALIGN_CENTER));
-      { Fl_Box* o = new Fl_Box(140, 43, 1, 25, "Options: ");
+      { Fl_Box* o = new Fl_Box(139, 43, 1, 25, "Options: ");
         o->labelfont(1);
         o->align(Fl_Align(FL_ALIGN_LEFT));
       } // Fl_Box* o
-      { tooltips_button = new Fl_Check_Button(138, 43, 113, 25, "Show Tooltips");
+      { tooltips_button = new Fl_Check_Button(137, 43, 113, 25, "Show Tooltips");
         tooltips_button->down_box(FL_DOWN_BOX);
         tooltips_button->labelsize(12);
         tooltips_button->callback((Fl_Callback*)cb_tooltips_button);
@@ -229,7 +246,7 @@
         tooltips_button->value(b);
         Fl_Tooltip::enable(b);
       } // Fl_Check_Button* tooltips_button
-      { completion_button = new Fl_Check_Button(138, 68, 186, 25, "Show Completion Dialogs");
+      { completion_button = new Fl_Check_Button(137, 68, 186, 25, "Show Completion Dialogs");
         completion_button->down_box(FL_DOWN_BOX);
         completion_button->labelsize(12);
         completion_button->callback((Fl_Callback*)cb_completion_button);
@@ -237,7 +254,7 @@
         fluid_prefs.get("show_completion_dialogs", b, 1);
         completion_button->value(b);
       } // Fl_Check_Button* completion_button
-      { openlast_button = new Fl_Check_Button(138, 93, 214, 25, "Open Previous File on Startup");
+      { openlast_button = new Fl_Check_Button(137, 93, 213, 25, "Open Previous File on Startup");
         openlast_button->down_box(FL_DOWN_BOX);
         openlast_button->labelsize(12);
         openlast_button->callback((Fl_Callback*)cb_openlast_button);
@@ -245,7 +262,7 @@
         fluid_prefs.get("open_previous_file", b, 0);
         openlast_button->value(b);
       } // Fl_Check_Button* openlast_button
-      { prevpos_button = new Fl_Check_Button(138, 118, 209, 25, "Remember Window Positions");
+      { prevpos_button = new Fl_Check_Button(137, 117, 209, 25, "Remember Window Positions");
         prevpos_button->down_box(FL_DOWN_BOX);
         prevpos_button->labelsize(12);
         prevpos_button->callback((Fl_Callback*)cb_prevpos_button);
@@ -253,7 +270,7 @@
         fluid_prefs.get("prev_window_pos", b, 1);
         prevpos_button->value(b);
       } // Fl_Check_Button* prevpos_button
-      { show_comments_button = new Fl_Check_Button(138, 143, 209, 25, "Show Comments in Browser");
+      { show_comments_button = new Fl_Check_Button(137, 141, 209, 25, "Show Comments in Browser");
         show_comments_button->down_box(FL_DOWN_BOX);
         show_comments_button->labelsize(12);
         show_comments_button->callback((Fl_Callback*)cb_show_comments_button);
@@ -262,7 +279,7 @@
       } // Fl_Check_Button* show_comments_button
       o->end();
     } // Fl_Group* o
-    { recent_spinner = new Fl_Spinner(140, 173, 40, 25, "# Recent Files: ");
+    { recent_spinner = new Fl_Spinner(138, 170, 40, 25, "# Recent Files: ");
       recent_spinner->labelfont(1);
       recent_spinner->callback((Fl_Callback*)cb_recent_spinner);
       recent_spinner->when(FL_WHEN_CHANGED);
@@ -271,7 +288,31 @@
       recent_spinner->maximum(10);
       recent_spinner->value(c);
     } // Fl_Spinner* recent_spinner
-    { Fl_Button* o = new Fl_Button(276, 205, 64, 25, "Close");
+    { Fl_Group* o = new Fl_Group(10, 210, 337, 95);
+      o->box(FL_THIN_UP_BOX);
+      o->color(FL_DARK1);
+      { use_external_editor_button = new Fl_Check_Button(25, 218, 209, 22, "Use external editor?");
+        use_external_editor_button->down_box(FL_DOWN_BOX);
+        use_external_editor_button->labelsize(12);
+        use_external_editor_button->callback((Fl_Callback*)cb_use_external_editor_button);
+        fluid_prefs.get("use_external_editor", G_use_external_editor, 0);
+        use_external_editor_button->value(G_use_external_editor);
+      } // Fl_Check_Button* use_external_editor_button
+      { editor_command_input = new Fl_Input(25, 264, 305, 21, "Editor Command");
+        editor_command_input->tooltip("The editor command to open your external text editor.\nInclude any necessary \
+flags to ensure your editor does not background itself.\nExamples:\n    gvim -\
+f\n    gedit\n emacs");
+        editor_command_input->labelsize(10);
+        editor_command_input->textsize(10);
+        editor_command_input->callback((Fl_Callback*)cb_editor_command_input);
+        editor_command_input->align(Fl_Align(FL_ALIGN_TOP_LEFT));
+        editor_command_input->when(FL_WHEN_CHANGED);
+        fluid_prefs.get("external_editor_command", G_external_editor_command, "", sizeof(G_external_editor_command)-1);
+        editor_command_input->value(G_external_editor_command);
+      } // Fl_Input* editor_command_input
+      o->end();
+    } // Fl_Group* o
+    { Fl_Button* o = new Fl_Button(285, 320, 64, 25, "Close");
       o->tooltip("Close this dialog.");
       o->callback((Fl_Callback*)cb_Close1);
     } // Fl_Button* o
Index: fluid/Fl_Function_Type.cxx
===================================================================
--- fluid/Fl_Function_Type.cxx	(revision 11804)
+++ fluid/Fl_Function_Type.cxx	(working copy)
@@ -20,8 +20,10 @@
 #include <FL/Fl_Preferences.H>
 #include <FL/Fl_File_Chooser.H>
 #include "Fl_Type.h"
+#include "ExternalCodeEditor.h"
 #include <FL/fl_show_input.H>
 #include <FL/Fl_File_Chooser.H>
+#include "alignment_panel.h"
 #include "../src/flstring.h"
 #include <stdio.h>
 #include <stdlib.h>
@@ -434,6 +436,14 @@
 }
 
 void Fl_Code_Type::open() {
+  // Using an external code editor? Open it..
+  if ( G_use_external_editor && G_external_editor_command[0] ) {
+    const char *cmd = G_external_editor_command;
+    const char *code = name();
+    if ( editor_.open_editor(cmd, code) == 0 )
+      return;		// return if editor opened ok, fallthru to built-in if not
+  }
+  // Use built-in code editor..
   if (!code_panel) make_code_panel();
   const char *text = name();
   code_input->buffer()->text( text ? text : "" );
@@ -459,9 +469,21 @@
 
 Fl_Code_Type Fl_Code_type;
 
+void Fl_Code_Type::write() {
+  // Check for external editor changes. If so, load changes into ram
+  // and update mtime/sizes before writing code.
+  editor_.handle_changes();
+  Fl_Type::write();
+}
+
 void Fl_Code_Type::write_code1() {
+  // Check for external editor changes. If so, load changes into ram
+  // and update mtime/sizes before writing code.
+  editor_.handle_changes();
+
   const char* c = name();
   if (!c) return;
+
   const char *pch;
   const char *ind = indent();
   while( (pch=strchr(c,'\n')) )
Index: fluid/Makefile
===================================================================
--- fluid/Makefile	(revision 11804)
+++ fluid/Makefile	(working copy)
@@ -18,6 +18,7 @@
 
 CPPFILES = \
 	CodeEditor.cxx \
+	ExternalCodeEditor.cxx \
 	Fl_Function_Type.cxx \
 	Fl_Group_Type.cxx \
 	Fl_Menu_Type.cxx \
Index: fluid/alignment_panel.h
===================================================================
--- fluid/alignment_panel.h	(revision 11804)
+++ fluid/alignment_panel.h	(working copy)
@@ -27,6 +27,8 @@
 extern void load_history();
 extern void redraw_browser();
 extern int show_comments;
+extern int G_use_external_editor;
+extern char G_external_editor_command[512];
 extern int show_coredevmenus;
 extern struct Fl_Menu_Item *dbmanager_item;
 extern Fl_Preferences fluid_prefs;
@@ -73,6 +75,8 @@
 extern Fl_Check_Button *show_comments_button;
 #include <FL/Fl_Spinner.H>
 extern Fl_Spinner *recent_spinner;
+extern Fl_Check_Button *use_external_editor_button;
+extern Fl_Input *editor_command_input;
 Fl_Double_Window* make_settings_window();
 extern Fl_Menu_Item menu_scheme_choice[];
 extern Fl_Double_Window *shell_window;
Index: fluid/ExternalCodeEditor.cxx
===================================================================
--- fluid/ExternalCodeEditor.cxx	(revision 0)
+++ fluid/ExternalCodeEditor.cxx	(revision 0)
@@ -0,0 +1,5 @@
+#ifdef WIN32
+#include "ExternalCodeEditor_WIN32.cxx"
+#else
+#include "ExternalCodeEditor_UNIX.cxx"
+#endif
Index: fluid/ExternalCodeEditor_UNIX.cxx
===================================================================
--- fluid/ExternalCodeEditor_UNIX.cxx	(revision 0)
+++ fluid/ExternalCodeEditor_UNIX.cxx	(revision 0)
@@ -0,0 +1,433 @@
+#include <errno.h>      // errno
+#include <string.h>     // strerror()
+#include <sys/types.h>  // stat()..
+#include <sys/stat.h>
+#include <fcntl.h>      // open()..
+#include <sys/wait.h>   // waitpid()..
+#include <signal.h>     // kill()..
+
+#include <unistd.h>
+#include <stdlib.h>     // free()
+#include <stdio.h>      // snprintf()..
+
+#include <FL/Fl.H>      // Fl_Timeout_Handler..
+#include <FL/fl_ask.H>  // fl_alert()
+
+#include "ExternalCodeEditor_UNIX.h"
+
+// External code editor class for Unix
+
+extern int G_debug;     // defined in fluid.cxx
+
+// Static local data
+static int L_editors_open = 0;				// keep track of #editors open
+static Fl_Timeout_Handler L_update_timer_cb = 0;	// app's update timer callback
+
+// See if file exists
+static int is_file(const char *filename) {
+  struct stat buf;
+  if ( stat(filename, &buf) < 0 ) return(0);
+  return(S_ISREG(buf.st_mode) ? 1 : 0); // regular file?
+}
+
+// See if dir exists
+static int is_dir(const char *dirname) {
+  struct stat buf;
+  if ( stat(dirname, &buf) < 0 ) return(0);
+  return(S_ISDIR(buf.st_mode) ? 1 : 0); // a dir?
+}
+
+// CTOR
+ExternalCodeEditor::ExternalCodeEditor() {
+  pid_        = -1;
+  filename_   = 0;
+  file_mtime_ = 0;
+  file_size_  = 0;
+}
+
+// DTOR
+ExternalCodeEditor::~ExternalCodeEditor() {
+  if ( G_debug )
+    printf("ExternalCodeEditor() DTOR CALLED (this=%p, pid=%ld)\n",
+           (void*)this, (long)pid_);
+  kill_editor();    // Kill open editor, deletes tmp file
+  set_filename(0);  // free()s filename
+}
+
+// [Protected] Set the filename. Handles memory allocation/free
+//     If set to NULL, frees memory.
+//
+void ExternalCodeEditor::set_filename(const char *val) {
+  if ( filename_ ) free((void*)filename_);
+  filename_ = val ? strdup(val) : 0;
+}
+
+// [Public] Is editor running?
+int ExternalCodeEditor::is_editing() {
+  return( (pid_ != -1) ? 1 : 0 );
+}
+
+// [Protected] Kill the running editor (if any)
+//   Kills the editor, reaps the process, and removes the tmp file.
+//   The dtor calls this to ensure no editors remain running when fluid exits.
+//
+void ExternalCodeEditor::kill_editor() {
+  if ( G_debug ) printf("kill_editor() called: pid=%ld\n", (long)pid_);
+  if ( !is_editing() ) return;  // editor not running? return..
+  kill(pid_, SIGTERM);          // kill editor
+  int wcount = 0;
+  while ( pid_ != -1 ) {        // and wait for it to finish..
+    usleep(100000);             // 1/10th sec delay gives editor time to close itself
+    switch (reap_editor()) {
+      case -1:  // error
+        fl_alert("Can't seem to close editor of file: %s\n"
+                 "waitpid() returned: %s\n"
+                 "Please close editor and hit OK",
+                 filename_,
+                 strerror(errno));
+        continue;
+      case 0:   // process still running
+        if ( ++wcount > 3 ) {   // retry 3x with 1/10th delay before showing dialog
+          fl_alert("Can't seem to close editor of file: %s\n"
+                   "Please close editor and hit OK",
+                   filename_);
+        }
+        continue;
+      default:  // process reaped
+        if ( G_debug )
+	  printf("*** REAPED KILLED EXTERNAL EDITOR: PID %ld\n", (long)pid_);
+        pid_ = -1;
+        break;
+    }
+  }
+  return;
+}
+
+// [Static] Return editor's file size in bytes
+static size_t get_file_size(const char *filename) {
+  struct stat sbuf;
+  if ( stat(filename, &sbuf) < 0 ) {
+    fl_alert("ERROR: can't get file size for '%s': %s",
+      filename, strerror(errno));
+    return(0);
+  }
+  return sbuf.st_size;
+}
+
+// [Public] Handle if file changed since last check, and update records if so.
+//
+// OPTIONAL TODO:
+//   Ignore changes made within the last 2 seconds,
+//   to give editor time to fully write out the file.
+//
+// Returns:
+//     1 -- file changed, internal records updated
+//     0 -- file unchanged
+//    -1 -- error getting file info (strerror() has reason)
+//
+int ExternalCodeEditor::handle_changes() {
+  // Get current time/size info
+  struct stat sbuf;
+  if ( stat(filename_, &sbuf) < 0 ) return(-1);
+  time_t now_mtime = sbuf.st_mtime;
+  size_t now_size  = sbuf.st_size;
+  // Changed? Update records, return true
+  int changed = 0;
+  if ( now_mtime != file_mtime_ ) { changed = 1; file_mtime_ = now_mtime; }
+  if ( now_size  != file_size_  ) { changed = 1; file_size_  = now_size; }
+  return changed;
+}
+
+// [Public] Remove the tmp file (if it exists), and zero out filename/mtime/size
+// Returns:
+//    -1 -- on error (dialog is posted as to why)
+//     0 -- no file to remove
+//     1 -- file was removed
+//
+int ExternalCodeEditor::remove_tmpfile() {
+  const char *tmpfile = filename();
+  if ( !tmpfile ) return 0;
+  // Filename set? remove (if exists) and zero filename/mtime/size
+  if ( is_file(tmpfile) ) {
+    if ( G_debug ) printf("Removing tmpfile '%s'\n", tmpfile);
+    if ( remove(tmpfile) < 0 ) {
+      fl_alert("WARNING: Can't remove() '%s': %s", tmpfile, strerror(errno));
+      return -1;
+    }
+  }
+  set_filename(0);
+  file_mtime_ = 0;
+  file_size_  = 0;
+  return 1;
+}
+
+// [Static/Public] Return tmpdir name for this fluid instance.
+//     Returns pointer to static memory.
+//
+const char* ExternalCodeEditor::tmpdir_name() {
+  static char dirname[100];
+  snprintf(dirname, sizeof(dirname), "/tmp/.fluid-%ld", (long)getpid());
+  return dirname;
+}
+
+// [Static/Public] Clear the external editor's tempdir
+//    Static so that the main program can call it on exit to clean up.
+//
+void ExternalCodeEditor::tmpdir_clear() {
+  const char *tmpdir = tmpdir_name();
+  if ( is_dir(tmpdir) ) {
+    if ( G_debug ) printf("Removing tmpdir '%s'\n", tmpdir);
+    if ( rmdir(tmpdir) < 0 ) {
+      fl_alert("WARNING: Can't rmdir() '%s': %s", tmpdir, strerror(errno));
+    }
+  }
+}
+
+// [Protected] Creates temp dir (if doesn't exist) and returns the dirname
+// as a static string. Returns NULL on error, dialog shows reason.
+//
+const char* ExternalCodeEditor::create_tmpdir() {
+  const char *dirname = tmpdir_name();
+  if ( ! is_dir(dirname) ) {
+    if ( mkdir(dirname, 0777) < 0 ) {
+      fl_alert("can't create directory '%s': %s",
+        dirname, strerror(errno));
+      return NULL;
+    }
+  }
+  return dirname;
+}
+
+// [Protected] Returns temp filename in static buffer.
+//    Returns NULL if can't, posts dialog explaining why.
+//
+const char* ExternalCodeEditor::tmp_filename() {
+  static char path[512];
+  const char *tmpdir = create_tmpdir();
+  if ( !tmpdir ) return 0;
+  extern const char *code_file_name;	// fluid's global
+  const char *ext  = code_file_name;	// e.g. ".cxx"
+  snprintf(path, sizeof(path), "%s/%p%s", tmpdir, (void*)this, ext);
+  path[sizeof(path)-1] = 0;
+  return path;
+}
+
+// Save string 'code' to 'filename'
+// Returns:
+//    0 on success
+//   -1 on error (posts dialog with reason)
+//
+static int save_file(const char *filename, const char *code) {
+  int fd = open(filename, O_WRONLY|O_CREAT, 0666);
+  if ( fd == -1 ) {
+    fl_alert("ERROR: open() '%s': %s", filename, strerror(errno));
+    return -1;
+  }
+  ssize_t clen = strlen(code);
+  ssize_t count = write(fd, code, clen);
+  int ret = 0;
+  if ( count == -1 ) {
+    fl_alert("ERROR: write() '%s': %s", filename, strerror(errno));
+    ret = -1; // fallthru to close()
+  } else if ( count != clen ) {
+    fl_alert("ERROR: write() '%s': wrote only %lu bytes, expected %lu",
+             filename, (unsigned long)count, (unsigned long)clen);
+    ret = -1; // fallthru to close()
+  }
+  close(fd);
+  return(ret);
+}
+
+// [Protected] Convert string 's' to array of argv[], useful for execve()
+//     o 's' will be modified (words will be NULL separated)
+//     o argv[] will end up pointing to the words of 's'
+//     o Caller must free argv with: free(argv);
+//
+int ExternalCodeEditor::make_args(char *s,         // string containing words (gets trashed!)
+                                  int *aargc,      // pointer to argc
+                                  char ***aargv) { // pointer to argv
+  char *ss, **argv;
+  if ((argv=(char**)malloc(sizeof(char*) * (strlen(s)/2)))==NULL) {
+    return -1;
+  }
+  int t;
+  for(t=0; (t==0)?(ss=strtok(s," \t")):(ss=strtok(0," \t")); t++) {
+    argv[t] = ss;
+  }
+  argv[t] = 0;
+  aargv[0] = argv;
+  aargc[0] = t;
+  return(t);
+}
+
+// [Protected] Start editor in background (fork/exec)
+// Returns:
+//    >  0 on success, leaves editor child process running as 'pid_'
+//    > -1 on error, posts dialog with reason (child exits)
+//
+int ExternalCodeEditor::start_editor(const char *editor_cmd,
+                                     const char *filename) {
+  if ( G_debug ) printf("start_editor() cmd='%s', filename='%s'\n",
+                        editor_cmd, filename);
+  char cmd[1024];
+  snprintf(cmd, sizeof(cmd), "%s %s", editor_cmd, filename);
+  // Fork editor to background..
+  switch ( pid_ = fork() ) {
+    case -1:    // error
+      fl_alert("couldn't fork(): %s", strerror(errno));
+      return -1;
+    case 0: {   // child
+      // NOTE: OSX wants minimal code between fork/exec, see Apple TN2083
+      int nargs;
+      char **args;
+      make_args(cmd, &nargs, &args);
+      execvp(args[0], args);  // run command - doesn't return if succeeds
+      fl_alert("couldn't exec() '%s': %s", cmd, strerror(errno));
+      exit(1);
+    }
+    default:    // parent
+      L_editors_open++;
+      if ( G_debug )
+        printf("*** EDITOR STARTED: pid_=%ld #open=%d\n", (long)pid_, L_editors_open);
+      break;
+  }
+  return 0;
+}
+
+// [Public] Try to reap external editor process
+// Returns:
+//   -2 -- editor not open
+//   -1 -- waitpid() failed (errno has reason)
+//    0 -- process still running
+//   >0 -- process finished + reaped (value is pid)
+//         Handles removing tmpfile/zeroing file_mtime/file_size
+//
+pid_t ExternalCodeEditor::reap_editor() {
+  if ( !is_editing() ) return -2;
+  int status = 0;
+  pid_t wpid;
+  switch (wpid = waitpid(pid_, &status, WNOHANG)) {
+    case -1:    // waitpid() failed
+      return -1;
+    case 0:     // process didn't reap, still running
+      return 0;
+    default:    // process reaped
+      remove_tmpfile(); // also zeroes mtime/size
+      pid_ = -1;
+      L_editors_open--;
+      break;
+  }
+  if ( G_debug ) 
+    printf("*** EDITOR REAPED: pid=%ld #open=%d\n",
+      long(wpid), L_editors_open);
+  return wpid;
+}
+
+// [Public] Open external editor using 'editor_cmd' to edit 'code'
+// 'code' contains multiline code to be edited as a temp file.
+//
+// Returns:
+//   0 if succeeds
+//   -1 if can't open editor (already open, etc),
+//      errors were shown to user in a dialog
+//
+int ExternalCodeEditor::open_editor(const char *editor_cmd,
+                                    const char *code) {
+  // Make sure a temp filename exists
+  if ( !filename_ ) {
+    set_filename(tmp_filename());
+    if ( !filename_ ) return -1;
+  }
+  // See if tmpfile already exists or editor already open
+  if ( is_file(filename_) ) {
+    if ( is_editing() ) {
+      // See if editor recently closed but not reaped; try to reap
+      pid_t wpid = reap_editor();
+      switch (wpid) {
+        case -1:        // waitpid() failed
+          fl_alert("ERROR: waitpid() failed: %s\nfile='%s', pid=%ld",
+            strerror(errno), filename_, (long)pid_);
+          return -1;
+        case 0:         // process still running
+          fl_alert("ERROR: editor already open: '%s' (PID=%ld)",
+            filename_, (long)pid_);
+          return 0;
+        default:        // process reaped, wpid is pid reaped
+          if ( G_debug )
+            printf("*** REAPED EXTERNAL EDITOR: PID %ld\n", (long)wpid);
+          break;        // fall thru to open new editor instance
+      }
+      // Reinstate tmp filename (reap_editor() clears it)
+      set_filename(tmp_filename());
+    }
+  }
+  if ( save_file(filename_, code) < 0 ) {
+    return -1;  // errors were shown in dialog
+  }
+  // Update mtime/size from closed file
+  struct stat sbuf;
+  if ( stat(filename_, &sbuf) < 0 ) {
+    fl_alert("ERROR: can't stat('%s'): %s", filename_, strerror(errno));
+    return -1;
+  }
+  file_mtime_ = sbuf.st_mtime;
+  file_size_  = sbuf.st_size;
+  if ( start_editor(editor_cmd, filename_) < 0 ) { // open file in external editor
+    if ( G_debug ) printf("Editor failed to start\n");
+    return -1;  // errors were shown in dialog
+  }
+  if ( G_debug ) printf("Editor started as pid=%ld\n", (long)pid_);
+  // New editor opened -- start update timer (if not already)
+  if ( L_update_timer_cb && !Fl::has_timeout(L_update_timer_cb) ) {
+    if ( G_debug ) printf("--- Editor opened: STARTING UPDATE TIMER\n");
+    Fl::add_timeout(2.0, L_update_timer_cb);
+  }
+  return 0;
+}
+
+// [Public] Get the current source code from the editor session 
+// Returns:
+//    0 on success, 'code' will be a pointer to malloc()'ed string that caller must free().
+//   -1 on error, 'code' is NULL, dialog was posted to user explaining reason
+//
+int ExternalCodeEditor::get_code(const char **code) {
+  code[0] = 0;
+  size_t size = get_file_size(filename_);
+  if ( size == 0 ) return(-1);  // temp file disappeared?
+  int fd = open(filename_, O_RDONLY);
+  if ( fd < 0 ) {
+    fl_alert("ERROR: can't open '%s': %s", filename_, strerror(errno));
+    return -1;
+  }
+  char *buf = (char*)malloc(size + 1);
+  ssize_t count = read(fd, buf, size);
+  if ( count == -1 ) {
+    fl_alert("ERROR: read() %s: %s", filename_, strerror(errno));
+    free((void*)buf);
+    return -1;
+  }
+  if ( (long)count != (long)size ) {
+    fl_alert("WARNING: read() failed for %s:\nexpected %lu bytes, only got %lu",
+             filename_, (unsigned long)size, (unsigned long)count);
+  }
+  close(fd);
+  buf[count] = '\0';
+  code[0] = buf;        // return pointer to allocated buffer
+  return(0);
+}
+
+// [Static/Public] See if any external editors are open.
+//     Can be used by app's modify timer to determine if there are any editors
+//     to be checked..
+//
+int ExternalCodeEditor::editors_open() {
+  return L_editors_open;
+}
+
+// [Public/Static] Set app's external editor update timer callback
+//   This is the app's callback callback we start while editors are open,
+//   and stop when all editors are closed.
+//
+void ExternalCodeEditor::set_update_timer_callback(Fl_Timeout_Handler cb) {
+  L_update_timer_cb = cb;
+}
Index: fluid/Fl_Type.cxx
===================================================================
--- fluid/Fl_Type.cxx	(revision 11804)
+++ fluid/Fl_Type.cxx	(working copy)
@@ -756,6 +756,7 @@
 int Fl_Type::is_menu_button() const {return 0;}
 int Fl_Type::is_group() const {return 0;}
 int Fl_Type::is_window() const {return 0;}
+int Fl_Type::is_code() const {return 0;}
 int Fl_Type::is_code_block() const {return 0;}
 int Fl_Type::is_decl_block() const {return 0;}
 int Fl_Type::is_comment() const {return 0;}
Index: fluid/fluid.cxx
===================================================================
--- fluid/fluid.cxx	(revision 11804)
+++ fluid/fluid.cxx	(working copy)
@@ -83,6 +83,9 @@
 #endif // HAVE_LIBPNG && HAVE_LIBZ
 }
 
+//
+// Globals..
+//
 static Fl_Help_Dialog *help_dialog = 0;
 
 Fl_Preferences	fluid_prefs(Fl_Preferences::USER, "fltk.org", "fluid");
@@ -91,6 +94,9 @@
 int snap = 1;
 int show_guides = 1;
 int show_comments = 1;
+int G_use_external_editor = 0;
+int G_debug = 0;
+char G_external_editor_command[512];
 int show_coredevmenus = 1;
 
 // File history info...
@@ -185,6 +191,40 @@
   return name[which];
 }
 
+// Timer to watch for external editor modifications
+//
+//    If one or more external editors are open, see if files they're
+//    editing were modified. If so: load changes, update our records
+//    of file's size + timestamp, and change 'modified' state of fluid.
+//
+static void external_editor_timer(void*) {
+  int editors_open = ExternalCodeEditor::editors_open();
+  if ( editors_open > 0 ) {
+    if ( G_debug ) printf("--- TIMER --- External editors open=%d\n", editors_open);
+    // Walk tree looking for files modified by external editors.
+    int modified = 0;
+    for (Fl_Type *p = Fl_Type::first; p; p = p->next) {
+      if ( p->is_code() ) {
+        Fl_Code_Type *code = (Fl_Code_Type*)p;
+        // Code changed by external editor? If so, reload file to ram,
+        // adjust time stamp records, and count modified files.
+        if ( code->handle_changes() ) {
+          modified++;
+        }
+        if ( code->is_editing() ) {     // editor open?
+          code->reap_editor();          // Try to reap it; maybe it recently closed
+        }
+      }
+    }
+    if ( modified ) set_modflag(1);                  // files modified? bump modify flag
+    Fl::repeat_timeout(2.0, external_editor_timer);  // keep timer running
+  } else {
+    // No editors running? Disable update timer until one opens again
+    Fl::remove_timeout(external_editor_timer);
+    if ( G_debug ) printf("--- REMOVING UPDATE TIMEOUT\n");
+  }
+}
+
 void save_cb(Fl_Widget *, void *v) {
   Fl_Native_File_Chooser fnfc;
   const char *c = filename;
@@ -371,6 +411,10 @@
 }
 
 void exit_cb(Fl_Widget *,void *) {
+
+  // Stop any external editor update timers
+  Fl::remove_timeout(external_editor_timer);
+
   if (modflag)
     switch (fl_choice("Do you want to save changes to this user\n"
                       "interface before exiting?", "Cancel",
@@ -404,6 +448,12 @@
 
   undo_clear();
 
+  // Destroy tree
+  //    Doing so causes dtors to automatically close all external editors
+  //    and cleans up editor tmp files. Then remove fluid tmpdir /last/.
+  delete_all();
+  ExternalCodeEditor::tmpdir_clear();
+
   exit(0);
 }
 
@@ -1711,7 +1761,6 @@
 }
 #endif
 
-
 int main(int argc,char **argv) {
   int i = 1;
   
@@ -1800,6 +1849,9 @@
   signal(SIGINT,sigint);
 #endif
 
+  // Set (but do not start) timer callback for external editor updates
+  ExternalCodeEditor::set_update_timer_callback(external_editor_timer);
+
   grid_cb(horizontal_input, 0); // Makes sure that windows get snap params...
 
 #ifdef WIN32
Index: fluid/ExternalCodeEditor_WIN32.cxx
===================================================================
--- fluid/ExternalCodeEditor_WIN32.cxx	(revision 0)
+++ fluid/ExternalCodeEditor_WIN32.cxx	(revision 0)
@@ -0,0 +1,562 @@
+#include <stdio.h>              // snprintf()
+
+#include <FL/Fl.H>      // Fl_Timeout_Handler..
+#include <FL/fl_ask.H>  // fl_alert()
+
+#include "ExternalCodeEditor_WIN32.h"
+
+//
+// External code editor class for Windows
+//
+
+extern int G_debug;     // defined in fluid.cxx
+
+// Static local data
+static int L_editors_open = 0;                          // keep track of #editors open
+static Fl_Timeout_Handler L_update_timer_cb = 0;        // app's update timer callback
+
+// [Static] Get error message string for last failed WIN32 function.
+// Returns a string pointing to static memory.
+//
+// TODO: Is more code needed here to convert returned string to utf8? -erco
+//
+static const char *get_ms_errmsg() {
+  static char emsg[1024];
+  DWORD flags = 0;
+  flags |= FORMAT_MESSAGE_ALLOCATE_BUFFER;
+  flags |= FORMAT_MESSAGE_IGNORE_INSERTS;
+  flags |= FORMAT_MESSAGE_FROM_SYSTEM;
+  DWORD lerr = GetLastError();
+  LPSTR MessageBuffer;
+  DWORD size = FormatMessageA(
+               flags,
+               0,
+               lerr,
+               MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // default language
+               (LPSTR)&MessageBuffer,
+               0,
+               NULL
+               );
+  if ( size != 0 ) {
+    _snprintf(emsg, sizeof(emsg), "%s", MessageBuffer);
+        emsg[sizeof(emsg)-1] = 0;
+    LocalFree(MessageBuffer);    // Free the buffer allocated by the system
+  } else {
+        sprintf(emsg, "Error Code %ld", long(lerr));
+  }
+  return emsg;
+}
+
+// See if file exists
+static int is_file(const char *filename) {
+  DWORD att = GetFileAttributesA(filename);
+  if (att == INVALID_FILE_ATTRIBUTES) return 0;
+  if ( (att & FILE_ATTRIBUTE_DIRECTORY) == 0 ) return 1;	// not a dir == file
+  return 0;
+}
+
+// See if dir exists
+static int is_dir(const char *dirname) {
+  DWORD att = GetFileAttributesA(dirname);
+  if (att == INVALID_FILE_ATTRIBUTES) return 0;
+  if (att & FILE_ATTRIBUTE_DIRECTORY) return 1;
+  return 0;
+}
+
+// CTOR
+ExternalCodeEditor::ExternalCodeEditor() {
+  memset(&pinfo_, 0, sizeof(pinfo_));
+  filename_   = 0;
+  memset(&file_mtime_, 0, sizeof(file_mtime_));
+  memset(&file_size_, 0, sizeof(file_size_));
+}
+
+// DTOR
+ExternalCodeEditor::~ExternalCodeEditor() {
+  if ( G_debug )
+    printf("ExternalCodeEditor() DTOR CALLED (this=%p, pid=%ld)\n",
+           (void*)this, (long)(pinfo_.dwProcessId));
+  kill_editor();    // Kill open editor, deletes tmp file
+  set_filename(0);  // free()s filename
+}
+
+// Set the filename. Handles memory allocation/free
+//     If set to NULL, frees memory.
+//
+void ExternalCodeEditor::set_filename(const char *val) {
+  if ( filename_ ) free((void*)filename_);
+  filename_ = val ? strdup(val) : 0;
+}
+
+// Is editor running?
+int ExternalCodeEditor::is_editing() {
+  return( (pinfo_.dwProcessId != 0) ? 1 : 0 );
+}
+
+// Send WM_CLOSE to a window
+static BOOL CALLBACK terminate_app_enum(HWND hwnd, LPARAM lParam) {
+  DWORD dwID;
+  GetWindowThreadProcessId(hwnd, &dwID);
+  if (dwID == (DWORD)lParam) PostMessage(hwnd, WM_CLOSE, 0, 0);
+  return TRUE;
+}
+
+int terminate_app(DWORD dwPID, DWORD dwTimeout) {
+  DWORD   dwRet ;
+  HANDLE hProc = OpenProcess(SYNCHRONIZE|PROCESS_TERMINATE, FALSE, dwPID);
+  if(hProc == NULL) return -1;
+  // terminate_app_enum() posts WM_CLOSE to all windows whose PID matches dwPID
+  EnumWindows((WNDENUMPROC)terminate_app_enum, (LPARAM) dwPID);
+  // Wait on the handle. If it signals, great. If it times out, kill it.
+  int ret = 0;
+  if ( WaitForSingleObject(hProc, dwTimeout) != WAIT_OBJECT_0 ) {
+        if ( G_debug ) {
+          printf("WARNING: sent WIN_CLOSE, but wait timed out.. trying TerminateProcess\n");
+        }
+    if ( TerminateProcess(hProc, 0) == 0 ) {
+      if ( G_debug ) {
+                printf("ERROR: TerminateProcess() for pid=%ld failed: %s\n",
+                       long(dwPID), get_ms_errmsg());
+          }
+          ret = -1;
+        } else {
+      // TerminateProcess succeeded
+      ret = 0;
+        }
+  } else {
+        // Wait succeeded
+    ret = 0;
+  }
+  CloseHandle(hProc);
+  return ret;
+}
+
+// [Protected] Kill the running editor (if any)
+//   Kills the editor, reaps the process, and removes the tmp file.
+//   The dtor calls this to ensure no editors remain running when fluid exits.
+//
+void ExternalCodeEditor::kill_editor() {
+  if ( G_debug )
+    printf("kill_editor() called: pid=%ld\n", (long)pinfo_.dwProcessId);
+  if ( !is_editing() ) return;  // editor not running? return..
+  switch ( terminate_app(pinfo_.dwProcessId, 100) ) {  // kill editor
+    case -1: { // error
+      fl_alert("Can't seem to close editor of file: %s\n"
+               "Please close editor and hit OK", filename_);
+      break;
+    }
+    case 0: {  // success -- process reaped
+      DWORD pid = pinfo_.dwProcessId;      // save pid
+      remove_tmpfile();                    // also zeroes mtime/size
+      CloseHandle(pinfo_.hProcess);
+      CloseHandle(pinfo_.hThread);
+      memset(&file_mtime_, 0, sizeof(file_mtime_));
+      memset(&file_size_, 0, sizeof(file_size_));
+      memset(&pinfo_, 0, sizeof(pinfo_));
+      set_filename(0);
+      L_editors_open--;
+      if ( G_debug ) printf("*** kill_editor() REAPED: pid=%ld #open=%d\n",
+                            long(pid), L_editors_open);
+      break;
+    }
+  }
+  return;
+}
+
+
+// Handle if file changed since last check, and update records if so.
+//
+// OPTIONAL TODO:
+//   Ignore changes made within the last 2 seconds,
+//   to give editor time to fully write out the file.
+//
+// Returns:
+//     1 -- file changed, internal records updated
+//     0 -- file unchanged
+//    -1 -- error getting file info (get_ms_errmsg() has reason)
+//
+int ExternalCodeEditor::handle_changes() {
+  // Sigh, have to open file to get file time/size :/
+  HANDLE fh = CreateFile(filename_,
+                         GENERIC_READ,
+                         FILE_SHARE_READ,
+                         NULL,
+                         OPEN_EXISTING,
+                         0,
+                         NULL);
+  if ( fh == INVALID_HANDLE_VALUE ) return -1;
+  LARGE_INTEGER fsize;
+  // Get file size
+  if ( GetFileSizeEx(fh, &fsize) == 0 ) {
+    DWORD err = GetLastError();
+    CloseHandle(fh); 
+    SetLastError(err);  // return error from GetFileSizeEx(), not CloseHandle()
+    return -1;
+  }
+  // Get file time
+  FILETIME ftCreate, ftAccess, ftWrite;
+  if ( GetFileTime(fh, &ftCreate, &ftAccess, &ftWrite) == 0 ) {
+    DWORD err = GetLastError();
+    CloseHandle(fh); 
+    SetLastError(err);  // return error from GetFileTime(), not CloseHandle()
+    return -1;
+  }
+  CloseHandle(fh);
+  // OK, now see if anything changed..
+  int changed = 0;
+  if ( fsize.QuadPart != file_size_.QuadPart )
+    { changed = 1; file_size_ = fsize; }
+  if ( CompareFileTime(&ftWrite, &file_mtime_) != 0 )
+    { changed = 1; file_mtime_ = ftWrite; }
+  return changed;
+}
+
+// Remove the tmp file (if it exists), and zero out filename/mtime/size
+// Returns:
+//    -1 -- on error (dialog is posted as to why)
+//     0 -- no file to remove
+//     1 -- file was removed
+//
+int ExternalCodeEditor::remove_tmpfile() {
+  const char *tmpfile = filename();
+  if ( G_debug ) printf("remove_tmpfile() '%s'\n", tmpfile ? tmpfile : "(empty)");
+  if ( !tmpfile ) return 0;
+  // Filename set? remove (if exists) and zero filename/mtime/size
+  if ( is_file(tmpfile) ) {
+    if ( G_debug ) printf("Removing tmpfile '%s'\n", tmpfile);
+    if ( DeleteFile(tmpfile) == 0 ) {
+      fl_alert("WARNING: Can't DeleteFile() '%s': %s", tmpfile, get_ms_errmsg());
+      return -1;
+    }
+  } else {
+    if ( G_debug ) printf("remove_tmpfile(): is_file(%s) failed\n", tmpfile);
+  }
+  set_filename(0);
+  memset(&file_mtime_, 0, sizeof(file_mtime_));
+  memset(&file_size_, 0, sizeof(file_size_));
+  return 1;
+}
+
+// [STATIC] Return tmpdir name for this fluid instance.
+//     Returns pointer to static memory.
+//
+const char* ExternalCodeEditor::tmpdir_name() {
+  char tempdir[100];
+  if (GetTempPath(sizeof(tempdir), tempdir) == 0 ) {
+    strcpy(tempdir, "c:\\windows\\temp");      // fallback
+  }
+  static char dirname[100];
+  _snprintf(dirname, sizeof(dirname), "%s.fluid-%ld",
+    tempdir, (long)GetCurrentProcessId());
+  if ( G_debug ) printf("tmpdir_name(): '%s'\n", dirname);
+  return dirname;
+}
+
+// [STATIC] Clear the external editor's tempdir
+//    Static so that the main program can call it on exit to clean up.
+//
+void ExternalCodeEditor::tmpdir_clear() {
+  const char *tmpdir = tmpdir_name();
+  if ( is_dir(tmpdir) ) {
+    if ( G_debug ) printf("Removing tmpdir '%s'\n", tmpdir);
+    if ( RemoveDirectory(tmpdir) == 0 ) {
+      fl_alert("WARNING: Can't RemoveDirectory() '%s': %s",
+               tmpdir, get_ms_errmsg());
+    }
+  }
+}
+
+// Creates temp dir (if doesn't exist) and returns the dirname
+// as a static string. Returns NULL on error, dialog shows reason.
+//
+const char* ExternalCodeEditor::create_tmpdir() {
+  const char *dirname = tmpdir_name();
+  if ( ! is_dir(dirname) ) {
+    if ( CreateDirectory(dirname,0) == 0 ) {
+      fl_alert("can't create directory '%s': %s",
+        dirname, get_ms_errmsg());
+      return NULL;
+    }
+  }
+  return dirname;
+}
+
+// Returns temp filename in static buffer.
+//    Returns NULL if can't, posts dialog explaining why.
+//
+const char* ExternalCodeEditor::tmp_filename() {
+  static char path[512];
+  const char *tmpdir = create_tmpdir();
+  if ( !tmpdir ) return 0;
+  extern const char *code_file_name;    // fluid's global
+  const char *ext  = code_file_name;    // e.g. ".cxx"
+  _snprintf(path, sizeof(path), "%s\\%p%s", tmpdir, (void*)this, ext);
+  path[sizeof(path)-1] = 0;
+  return path;
+}
+
+// Save string 'code' to 'filename', returning file's mtime/size
+// Returns:
+//    0 on success
+//   -1 on error (posts dialog with reason)
+//
+static int save_file(const char *filename,
+                     const char *code,
+                             FILETIME &file_mtime,            // return these since in win32 it's..
+                             LARGE_INTEGER &file_size) {  // ..efficient to get while file open
+  memset(&file_mtime, 0, sizeof(file_mtime));
+  memset(&file_size, 0, sizeof(file_size));
+  HANDLE fh = CreateFile(filename,
+                         GENERIC_WRITE,  // write only
+                         0,              // no sharing
+                         NULL,           // default security
+                         CREATE_ALWAYS,  // create flags
+                         FILE_ATTRIBUTE_NORMAL,
+                         NULL);          // no templates
+  if ( fh == INVALID_HANDLE_VALUE ) {
+    fl_alert("ERROR: couldn't create file '%s': %s",
+             filename, get_ms_errmsg());
+    return(-1);
+  }
+  DWORD clen = strlen(code);
+  DWORD count = 0;
+  int ret = 0;
+  if ( WriteFile(fh, code, clen, &count, NULL) == 0 ) {
+    fl_alert("ERROR: WriteFile() '%s': %s", filename, get_ms_errmsg());
+    ret = -1; // fallthru to CloseHandle()
+  } else if ( count != clen ) {
+    fl_alert("ERROR: WriteFile() '%s': wrote only %lu bytes, expected %lu",
+             filename, (unsigned long)count, (unsigned long)clen);
+    ret = -1; // fallthru to CloseHandle()
+  }
+  // Get mtime/size before closing
+  {
+    FILETIME ftCreate, ftAccess, ftWrite;
+    if ( GetFileSizeEx(fh, &file_size) == 0 ) {
+      fl_alert("ERROR: save_file(%s): GetFileSizeEx() failed: %s\n",
+               filename, get_ms_errmsg());
+    }
+    if ( GetFileTime(fh, &ftCreate, &ftAccess, &ftWrite) == 0 ) {
+      fl_alert("ERROR: save_file(%s): GetFileTime() failed: %s\n",
+               filename, get_ms_errmsg());
+    }
+    file_mtime = ftWrite;
+  }
+  // Close, done
+  CloseHandle(fh);
+  return(ret);
+}
+
+// Start editor
+// Returns:
+//    >  0 on success, leaves editor child process running
+//    > -1 on error, posts dialog with reason (child exits)
+//
+int ExternalCodeEditor::start_editor(const char *editor_cmd,
+                                     const char *filename) {
+  if ( G_debug ) printf("start_editor() cmd='%s', filename='%s'\n",
+                        editor_cmd, filename);
+  // CREATE FLAGS
+  int createflags = 0;
+
+  // STARTUP INFO
+  STARTUPINFO sinfo;
+  memset(&sinfo, 0, sizeof(sinfo));
+  sinfo.cb          = sizeof(sinfo);
+  sinfo.dwFlags     = 0;
+  sinfo.wShowWindow = 0;
+#define NO_CONSOLE_WINDOW 1
+#ifdef NO_CONSOLE_WINDOW
+  // No console window (what we want)
+  sinfo.dwFlags     |= STARTF_USESHOWWINDOW;
+  sinfo.wShowWindow |= SW_HIDE;
+  createflags       |= 0;
+#else
+  // Make console window (might be useful for debugging..)
+  sinfo.dwFlags     |= STARTF_USESHOWWINDOW;
+  sinfo.wShowWindow |= SW_SHOW;          // use SW_SHOWMINIMIZED instead to minimize
+  createflags       |= CREATE_NEW_CONSOLE;
+#endif
+
+  // PROCESS INFO
+  memset(&pinfo_, 0, sizeof(pinfo_));
+
+  // COMMAND
+  char cmd[1024];
+  _snprintf(cmd, sizeof(cmd), "%s %s", editor_cmd, filename);
+
+  // START EDITOR
+  if (CreateProcess(NULL,               // app name
+                    (char*)cmd,         // command to exec
+                    NULL,               // secure attribs
+                    NULL,               // thread secure attribs
+                    NULL,               // handle inheritence
+                    NULL,               // creation flags
+                    NULL,               // environ block
+                    NULL,               // current dir
+                    &sinfo,             // startup info
+                    &pinfo_) == 0 ) {   // process info
+    fl_alert("CreateProcess() failed to start '%s': %s",
+             cmd, get_ms_errmsg());
+    return(-1);
+  }
+  L_editors_open++;
+  if ( G_debug )
+    printf("*** EDITOR STARTED: pid_=%ld #open=%d\n",
+           (long)pinfo_.dwProcessId, L_editors_open);
+  return 0;
+}
+
+// Try to reap external editor process
+// Returns:
+//   -2 -- editor not open
+//   -1 -- WaitForSingleObject() failed (get_ms_errmsg() has reason)
+//    0 -- process still running
+//   >0 -- process finished + reaped (value is pid)
+//         Handles removing tmpfile/zeroing file_mtime/file_size
+//
+DWORD ExternalCodeEditor::reap_editor() {
+  if ( pinfo_.dwProcessId == 0 ) return -2;
+  int err;
+  DWORD msecs_wait = 50;   // .05 sec
+  switch ( err = WaitForSingleObject(pinfo_.hProcess, msecs_wait ) ) {
+    case WAIT_TIMEOUT: {   // process didn't reap, still running
+      return 0;
+    }
+    case WAIT_OBJECT_0: {  // reaped
+      DWORD pid = pinfo_.dwProcessId;      // save pid
+      remove_tmpfile();                    // also zeroes mtime/size
+      CloseHandle(pinfo_.hProcess);        // close process handle
+      CloseHandle(pinfo_.hThread);         // close thread handle
+      memset(&pinfo_, 0, sizeof(pinfo_));  // clear pinfo_
+      L_editors_open--;
+      if ( G_debug ) printf("*** EDITOR REAPED: pid=%ld #open=%d\n",
+                            long(pid), L_editors_open);
+      return pid;
+    }
+    case WAIT_FAILED: {    // failed
+      return -1;
+    }
+  }
+  return -1;               // any other return unexpected
+}
+
+// [Public] Open external editor using 'editor_cmd' to edit 'code'
+// 'code' contains multiline code to be edited as a temp file.
+//
+// Returns:
+//   0 if succeeds
+//   -1 if can't open editor (already open, etc),
+//      errors were shown to user in a dialog
+//
+int ExternalCodeEditor::open_editor(const char *editor_cmd,
+                                    const char *code) {
+  // Make sure a temp filename exists
+  if ( !filename_ ) {
+    set_filename(tmp_filename());
+    if ( !filename_ ) return -1;
+  }
+  // See if tmpfile already exists or editor already open
+  if ( is_file(filename_) ) {
+    if ( is_editing() ) {
+      // See if editor recently closed but not reaped; try to reap
+      DWORD wpid = reap_editor();
+      switch (wpid) {
+        case -1:        // wait failed
+          fl_alert("ERROR: WaitForSingleObject() failed: %s\nfile='%s', pid=%ld",
+            get_ms_errmsg(), filename_, long(pinfo_.dwProcessId));
+          return -1;
+        case 0:         // process still running
+          fl_alert("ERROR: editor already open: '%s' (PID=%ld)",
+            filename_, long(pinfo_.dwProcessId));
+          return 0;
+        default:        // process reaped, wpid is pid reaped
+          if ( G_debug )
+            printf("*** REAPED EXTERNAL EDITOR: PID %ld\n", long(wpid));
+          break;        // fall thru to open new editor instance
+      }
+      // Reinstate tmp filename (reap_editor() clears it)
+      set_filename(tmp_filename());
+    }
+  }
+  // Save code to tmpfile, getting mtime/size
+  if ( save_file(filename_, code, file_mtime_, file_size_) < 0 ) {
+    return -1;  // errors were shown in dialog
+  }
+  if ( start_editor(editor_cmd, filename_) < 0 ) { // open file in external editor
+    if ( G_debug ) printf("Editor failed to start\n");
+    return -1;  // errors were shown in dialog
+  }
+  if ( G_debug ) printf("Editor started as pid=%ld\n", (long)pinfo_.dwProcessId);
+  // New editor opened -- start update timer (if not already)
+  if ( L_update_timer_cb && !Fl::has_timeout(L_update_timer_cb) ) {
+    if ( G_debug ) printf("--- Editor opened: STARTING UPDATE TIMER\n");
+    Fl::add_timeout(2.0, L_update_timer_cb);
+  }
+  return 0;
+}
+
+// [Public] Get the current source code from the editor session 
+// Returns:
+//    0 on success, 'code' will be a pointer to malloc()'ed string that caller must free().
+//   -1 on error, 'code' is NULL, dialog was posted to user explaining reason
+//
+int ExternalCodeEditor::get_code(const char **code) {
+  code[0] = 0;
+  if ( G_debug ) printf("getcode() from %s\n", filename_);
+  HANDLE fh = CreateFile(filename_,
+                         GENERIC_READ,
+                         FILE_SHARE_READ,
+                         NULL,
+                         OPEN_EXISTING,
+                         FILE_ATTRIBUTE_NORMAL,
+                         NULL);
+  if ( fh == INVALID_HANDLE_VALUE ) {
+    fl_alert("ERROR: Could not open '%s' for reading: %s",
+             filename(), get_ms_errmsg());
+    return -1;
+  }
+  LARGE_INTEGER size;
+  if ( GetFileSizeEx(fh, &size) == 0 ) {
+    fl_alert("ERROR: Could not GetFileSizeEx() for '%s': %s",
+             filename(), get_ms_errmsg());
+    CloseHandle(fh);
+    return -1;
+  }
+  if ( size.QuadPart == 0 ) return(0);  // file is empty?
+  char *buf = (char*)malloc(size.QuadPart + 1);
+  DWORD count;
+  if ( ReadFile(fh, buf, size.QuadPart, &count, 0) == 0 ) {
+    fl_alert("ERROR: ReadFile() failed for %s: %s",
+             filename(), get_ms_errmsg());
+    free((void*)buf);
+    buf[0] = 0;
+    CloseHandle(fh);
+    return(-1);
+  }
+  if ( count != size.QuadPart ) {
+    fl_alert("WARNING: ReadFile() failed for %s:\nexpected %lu bytes, got %lu",
+             filename(), long(size.QuadPart), (unsigned long)count);
+  }
+  CloseHandle(fh);
+  buf[count] = '\0';
+  code[0] = buf;        // return pointer to allocated buffer
+  return(0);
+}
+
+// [Static/Public] See if any external editors are open.
+//     Can be used by app's modify timer to determine if there are any editors
+//     to be checked..
+//
+int ExternalCodeEditor::editors_open() {
+  return L_editors_open;
+}
+
+// [Public/Static] Set app's external editor update timer callback
+//   This is the app's callback callback we start while editors are open,
+//   and stop when all editors are closed.
+//
+void ExternalCodeEditor::set_update_timer_callback(Fl_Timeout_Handler cb) {
+  L_update_timer_cb = cb;
+}
+
Index: fluid/ExternalCodeEditor.h
===================================================================
--- fluid/ExternalCodeEditor.h	(revision 0)
+++ fluid/ExternalCodeEditor.h	(revision 0)
@@ -0,0 +1,10 @@
+#ifndef _EXTCODEEDITOR_H
+#define _EXTCODEEDITOR_H
+
+#ifdef WIN32
+#include "ExternalCodeEditor_WIN32.h"
+#else
+#include "ExternalCodeEditor_UNIX.h"
+#endif
+
+#endif
Index: fluid/ExternalCodeEditor_UNIX.h
===================================================================
--- fluid/ExternalCodeEditor_UNIX.h	(revision 0)
+++ fluid/ExternalCodeEditor_UNIX.h	(revision 0)
@@ -0,0 +1,41 @@
+#include <errno.h>      // errno
+#include <string.h>     // strerror()
+
+#include <sys/types.h>  // stat()..
+#include <sys/stat.h>
+#include <unistd.h>
+
+//
+// External code editor class for Unix
+//
+//     Handles starting and keeping track of an external text editor,
+//     including process start, temp file creation, bookkeeping, killing..
+//
+class ExternalCodeEditor {
+  int pid_;
+  const char *filename_;
+  time_t file_mtime_;                   // last modify time of the file (used to determine if file changed)
+  size_t file_size_;                    // last file size (used to determine if changed)
+protected:
+  void kill_editor();
+  const char *create_tmpdir();
+  const char *tmp_filename();
+  int make_args(char *s, int *aargc, char ***argv);
+  int start_editor(const char *cmd, const char *filename);
+  void set_filename(const char *val);
+public:
+  ExternalCodeEditor();
+  ~ExternalCodeEditor();
+  int is_editing();
+  pid_t reap_editor();
+  const char *filename() { return filename_; }
+  int open_editor(const char *editor_cmd, const char *code);
+  int get_code(const char **code);
+  int handle_changes();
+  int remove_tmpfile();
+  // Public static methods
+  static const char* tmpdir_name();
+  static void tmpdir_clear();
+  static int editors_open();
+  static void set_update_timer_callback(Fl_Timeout_Handler);
+};
Index: ide/VisualC2010/fluid.vcxproj
===================================================================
--- ide/VisualC2010/fluid.vcxproj	(revision 11804)
+++ ide/VisualC2010/fluid.vcxproj	(working copy)
@@ -317,6 +317,7 @@
       <AdditionalIncludeDirectories Condition="'$(Configuration)|$(Platform)'=='Release|Win32'">%(AdditionalIncludeDirectories)</AdditionalIncludeDirectories>
       <PreprocessorDefinitions Condition="'$(Configuration)|$(Platform)'=='Release|Win32'">%(PreprocessorDefinitions)</PreprocessorDefinitions>
     </ClCompile>
+    <ClCompile Include="..\..\fluid\ExternalCodeEditor_WIN32.cxx" />
     <ClCompile Include="..\..\fluid\factory.cxx">
       <Optimization Condition="'$(Configuration)|$(Platform)'=='Debug Cairo|Win32'">Disabled</Optimization>
       <AdditionalIncludeDirectories Condition="'$(Configuration)|$(Platform)'=='Debug Cairo|Win32'">%(AdditionalIncludeDirectories)</AdditionalIncludeDirectories>
@@ -527,7 +528,11 @@
       <ReferenceOutputAssembly>false</ReferenceOutputAssembly>
     </ProjectReference>
   </ItemGroup>
+  <ItemGroup>
+    <ClInclude Include="..\..\fluid\ExternalCodeEditor.h" />
+    <ClInclude Include="..\..\fluid\ExternalCodeEditor_WIN32.h" />
+  </ItemGroup>
   <Import Project="$(VCTargetsPath)\Microsoft.Cpp.targets" />
   <ImportGroup Label="ExtensionTargets">
   </ImportGroup>
-</Project>
\ No newline at end of file
+</Project>
